#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun 16 08:15:47 2021

@author: papoteur
@license: GPL v3.0
"""

import sys
import os
import re
import ujson
from PyQt5.QtGui import QIcon, QStandardItemModel, QStandardItem
from PyQt5.QtCore import QCoreApplication, QDir, QLibraryInfo, QLocale, QSettings,  \
     QSize, Qt, QTranslator, QTimer
from PyQt5.QtWidgets import QAbstractItemView, QApplication, QCheckBox, QDialog, \
      QDialogButtonBox, QFileDialog, QHBoxLayout, QMenu, QPushButton, QSizePolicy, \
      QSystemTrayIcon, QTableWidget, QVBoxLayout, QWidget, QTableView 

MODEL_USER_PATH = os.path.expanduser('~/.config/vosk-models')
MODEL_GLOBAL_PATH = "/usr/share/vosk-model"
MODEL_LIST = 'model-list.json'
MODELS_URL = "https://alphacephei.com/vosk/models"


class Models(QStandardItemModel):
    def  __init__(self):
        QStandardItemModel.__init__(self, 0, 5)
        headers=[
            self.tr("Language"),
            self.tr("Name"),
            self.tr("Version"),
            self.tr("Size"),
            self.tr("Class"),
            ]
        i=0
        for label in headers:
            self.setHeaderData(i, Qt.Horizontal, label)
            i+=1

class DownloadPopup(QDialog):
    def __init__(self,  parent=None) -> None:
        super(QDialog, self).__init__(parent)
        self.list = Models()
        with open(os.path.join(MODEL_USER_PATH, MODEL_LIST)) as remote_list:
            self.remote_models = ujson.loads(remote_list.read())
        for model_data in sorted(self.remote_models, key=lambda item: item['lang_text']):
            if (model_data['obsolete'] != "true"):
                language_item = QStandardItem(model_data['lang_text'])
                name_item = QStandardItem(model_data['name'])
                size_item = QStandardItem(model_data['size_text'])
                version_item = QStandardItem(model_data['version'])
                class_item = QStandardItem(model_data['type'])
                self.list.appendRow([
                    language_item, 
                    name_item, 
                    version_item, 
                    size_item, 
                    class_item,
                    ])
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        vbox = QVBoxLayout(self)
        self.table = QTableView()
        self.table.setModel(self.list)
        vbox.addWidget(self.table)
        buttonBox = QDialogButtonBox(
            QDialogButtonBox.Save | QDialogButtonBox.Cancel
        )
        vbox.addWidget(buttonBox)
        self.table.horizontalHeader().setStretchLastSection(True)
        self.setLayout(vbox)
        self.table.resizeColumnsToContents()
        self.table.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.table.setSelectionMode(QAbstractItemView.SingleSelection)
        self.table.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.adjustSize()
        
    def cancel():
        self.close()

    def sizeHint(self):
        width = 0
        for i in range(self.list.columnCount()):
            width += self.table.columnWidth(i)

        width += self.table.verticalHeader().sizeHint().width()

        width += self.table.verticalScrollBar().sizeHint().width()
        width += self.table.frameWidth()*2

        return QSize(width,self.height())

def main() -> None:
    app = QApplication(sys.argv)
    # don't close application when closing setting window)
    app.setQuitOnLastWindowClosed(False)
    LOCAL_DIR = os.path.dirname(os.path.realpath(__file__))
    
    w = DownloadPopup()

    w.show()
    exit(app.exec_())


if __name__ == "__main__":
    main()
